import { ItemStack, system, world } from "@minecraft/server"
import { ActionFormData, ModalFormData } from "@minecraft/server-ui"

const Chests = new Set()

const armors = [
	{ id: 0, item: "", sound: "", event: "", name: "None", value: 0, attack: 3 },
	{ id: 1, item: "minecraft:iron_ingot", sound: "armor.equip_iron", event: "e:iron_armor", name: "§7Iron", value: 10, attack: 5 },
	{ id: 2, item: "minecraft:gold_ingot", sound: "armor.equip_gold", event: "e:gold_armor", name: "§gGold", value: 5, attack: 4 },
	{ id: 3, item: "minecraft:diamond", sound: "armor.equip_diamond", event: "e:diamond_armor", name: "§bDiamond", value: 20, attack: 7 },
	{ id: 4, item: "minecraft:netherite_ingot", sound: "armor.equip_netherite", event: "e:netherite_armor", name: "§uNetherite", value: 20, attack: 7 },
	{ id: 5, item: "minecraft:emerald", sound: "armor.equip_generic", event: "e:emerald_armor", name: "§aEmerald", value: 7, attack: 5 }
]

function getEquipment(entity, slot) {
	const equip = entity.getComponent("equippable")
	const item = equip?.getEquipment(slot)
	return item
}

function deductItemInHand(player) {
	if (player.matches({ gameMode: "creative" })) return
	const item = getEquipment(player, "Mainhand")
	if (!item) return
	const amount = item.amount
	if (amount > 1) item.amount--
	const equip = player.getComponent("equippable")
	equip.setEquipment("Mainhand", amount > 1 ? item : null)
}

function getRotation(dir) {
	if (dir == "north") return 0
	if (dir == "east") return 90
	if (dir == "south") return 180
	if (dir == "west") return 270
}

function equipArmor(entity, damagingEntity, item) {
	const armor = armors.findIndex(i => i.item == item)
	if (armor == -1) return
	if (armors[armor].id == entity.getProperty("leocabs:armor")) return chestSettings(damagingEntity, entity)
	deductItemInHand(damagingEntity)
	entity.setProperty("leocabs:armor", armors[armor].id)
	entity.triggerEvent(armors[armor].event)
	damagingEntity.playSound(armors[armor].sound, entity.location)
}
function chestSettings(player, chest) {
	const hp = Math.floor(chest.getComponent("health").currentValue);
	const maxHp = Math.floor(chest.getComponent("health").effectiveMax);
	const { name, value, attack } = armors[chest.getProperty("leocabs:armor")];
	const form = new ActionFormData()
		.title("Chest Buddy")
		.body(`§7Name: §r${chest.nameTag}\n§7Health: §r${hp}/${maxHp}\n§7Armor: §r${value} ${name}\n§7Attack: §r${attack}\n\n`)
		.button("Rename")
		.button("Behavior")
		.button("Kill");
	
	form.show(player).then((response) => {
		if (response.canceled) return;
		if (response.selection === 0) rename();
		if (response.selection === 1) behavior();
		if (response.selection === 2) {
			// Drop the chest's inventory items when killed
			const container = chest.getComponent("inventory").container;
			
			for (let i = 0; i < container.size; i++) {
				const item = container.getItem(i);
				if (item) {
					chest.dimension.spawnItem(item, chest.location);
				}
			}
			
			// Drop the chest itself as an item
			chest.dimension.spawnItem(new ItemStack("minecraft:chest"), chest.location);
			
			// Additional effects
			chest.dimension.spawnParticle("minecraft:egg_destroy_emitter", chest.location);
			chest.dimension.playSound("random.pop", chest.location);
			
			// Remove the chest entity
			chest.remove();
		}
	});
	
	// ... (rest of the rename and behavior functions)


	function rename() {
		const form = new ModalFormData()
			.title("Chest Buddy")
			.textField("Name:", "Leave blank to cancel")
		form.show(player).then((response) => {
			if (response.canceled) return
			const [NAME] = response.formValues
			if (!NAME) return chestSettings(player, chest)
			chest.nameTag = NAME
		})
	}

	function behavior() {
		const stroll = chest.getProperty("leocabs:stroll")
		const followOwner = chest.getProperty("leocabs:follow_owner")
		const protectOwner = chest.getProperty("leocabs:protect_owner")
		const form = new ModalFormData()
			.title("Chest Buddy")
			.toggle("§7Follow Owner:\nChest will follow you.\n\n", followOwner)
			.toggle("§7Stroll Around:\nChest randomly strolls around.\n\n", stroll)
			.toggle("§7Protect Owner:\nChest protects its owner from attackers.\n\n", protectOwner)
		form.show(player).then((response) => {
			if (response.canceled) return
			const [FOLLOW_OWNER, STROLL, PROTECT] = response.formValues
			if (FOLLOW_OWNER) {
				chest.triggerEvent("e:follow_owner")
			} else {
				chest.triggerEvent("e:unfollow_owner")
			}
			if (STROLL) {
				chest.triggerEvent("e:activate_random_stoll")
			} else {
				chest.triggerEvent("e:deactivate_random_stoll")
			}
			if (PROTECT) {
				chest.triggerEvent("e:protect_owner")
			} else {
				chest.triggerEvent("e:unprotect_owner")
			}
		})
	}
}

world.afterEvents.entitySpawn.subscribe((e) => {
	if (e.entity.typeId != "leocabs:chest_buddy") return
	const chest = e.entity
	Chests.add(chest)
	system.runTimeout(() => {
		if (!chest.isValid()) return
		const tameable = chest.getComponent("minecraft:tameable")
		if (!chest.owner || !tameable || tameable.isTamed) return
		tameable.tame(chest.owner)
		chest.nameTag = `${chest.owner.name}'s Chest Buddy`
		chest.setDynamicProperty("leocabs:owner", chest.owner.id)
		const items = chest.container
		system.runTimeout(() => {
			for (let i = 0; i < 27; i++) {
				const container = chest.getComponent("inventory").container
				const item = items[i]
				if (!item) container.setItem(i, null)
				container.setItem(i, item)
			}
		}, 1)
	}, 34)
})

world.afterEvents.entityLoad.subscribe((e) => {
	if (e.entity.typeId != "leocabs:chest_buddy") return
	Chests.add(e.entity)
})

world.beforeEvents.playerInteractWithBlock.subscribe((e) => {
	const { player, block } = e
	const item = getEquipment(player, "Mainhand")
	if (!item) return
	if (block.typeId != "minecraft:chest" || item.typeId != "leocabs:buddy_key" || player.isSneaking) return
	const direction = block.permutation.getState("minecraft:cardinal_direction")
	const container = block.getComponent("inventory").container
	e.cancel = true
	system.run(() => {
		const chestContainer = []
		for (let i = 0; i < 27; i++) {
			const item = container.getItem(i)
			if (!item) {
				chestContainer[i] = null
			} else {
				container.setItem(i, null)
				chestContainer[i] = item
			}
		}
		block.setType("minecraft:air")
		block.dimension.spawnParticle("minecraft:egg_destroy_emitter", block.bottomCenter())
		const chest = block.dimension.spawnEntity("leocabs:chest_buddy", block.bottomCenter())
		chest.setProperty("leocabs:spawn_rotation", getRotation(direction))
		chest.setRotation({ x: 0, y: getRotation(direction) - 180 })
		chest.owner = player
		chest.container = chestContainer
		deductItemInHand(player)
	})
})

world.beforeEvents.playerInteractWithEntity.subscribe((e) => {
	const { player, target } = e
	if (target.typeId != "leocabs:chest_buddy") return
	if (target.getDynamicProperty("leocabs:owner") != player.id) return
	if (!player.isSneaking) {
		system.run(() => {
			target.triggerEvent("e:freeze")
			target.setProperty("leocabs:is_frozen", true)
		})
	} else {
		e.cancel = true
		system.run(() => {
			const item = getEquipment(player, "Mainhand")?.typeId
			const armorEquip = armors.map(a => a.item).includes(item)
			if (armorEquip) return equipArmor(target, player, item)
			const health = target.getComponent("minecraft:health")
			const hp = health.currentValue
			const hpMax = health.effectiveMax
			if (!item?.includes("planks") || hp >= hpMax) return chestSettings(player, target)
			health.setCurrentValue(Math.min(hp + 4, hpMax))
			deductItemInHand(player)
			target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
			target.dimension.spawnParticle("leocabs:heal", target.location)
		})
	}
})